<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:template="http://www.boddie.org.uk/ns/xmltools/template"
                xmlns:dyn="http://exslt.org/dynamic"
                extension-element-prefixes="dyn"
                version="1.0">

  <xsl:param name="database"/>

  <xsl:template match="configuration">
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <!-- Complete all multiple choice fields. -->
      <details>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-element" select="details/base-system[1]"/>
          <xsl:with-param name="element-name">base-system</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-element" select="details/cpu[1]"/>
          <xsl:with-param name="control-element" select="details/base-system[1]"/>
          <xsl:with-param name="element-name">cpu</xsl:with-param>
          <xsl:with-param name="control-element-name">base-system</xsl:with-param>
          <xsl:with-param name="control-attribute-name">cpu-socket</xsl:with-param>
        </xsl:call-template>
      </details>
      <memory>
        <xsl:for-each select="memory/memory-unit">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-element" select="."/>
            <xsl:with-param name="element-name">memory-unit</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </memory>
      <hard-disks>
        <xsl:for-each select="hard-disks/hard-disk">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-element" select="."/>
            <xsl:with-param name="element-name">hard-disk</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </hard-disks>
      <storage>
        <xsl:for-each select="storage/storage-unit">
          <xsl:call-template name="complete-element">
            <xsl:with-param name="found-element" select="."/>
            <xsl:with-param name="element-name">storage-unit</xsl:with-param>
          </xsl:call-template>
        </xsl:for-each>
      </storage>
      <peripherals>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-element" select="peripherals/keyboard[1]"/>
          <xsl:with-param name="element-name">keyboard</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-element" select="peripherals/mouse[1]"/>
          <xsl:with-param name="element-name">mouse</xsl:with-param>
        </xsl:call-template>
        <xsl:call-template name="complete-element">
          <xsl:with-param name="found-element" select="peripherals/screen[1]"/>
          <xsl:with-param name="element-name">screen</xsl:with-param>
        </xsl:call-template>
      </peripherals>
    </xsl:copy>
  </xsl:template>

  <xsl:template name="complete-element">
    <!-- Input parameters. -->
    <xsl:param name="found-element"/>
    <xsl:param name="control-element"/>
    <!-- Database parameters. -->
    <xsl:param name="element-name"/>
    <xsl:param name="control-element-name"/>
    <xsl:param name="control-attribute-name"/>
    <!-- Find the database entries. -->
    <xsl:variable name="element-enum-name"><xsl:value-of select="$element-name"/>-enum</xsl:variable>
    <xsl:variable name="control-element-enum-name"><xsl:value-of select="$control-element-name"/>-enum</xsl:variable>
    <!-- Find whether the input values are set. -->
    <xsl:variable name="has-set-values" select="$found-element[@value]"/>
    <xsl:variable name="has-control-values" select="$control-element and $control-element[@value]"/>
    <!-- Get the database element for the element being prepared. -->
    <xsl:variable name="database-element" select="$database/database/*[local-name() = $element-name]"/>
    <!-- Get the database element for the control element. -->
    <xsl:variable name="database-control-element" select="$database/database/*[local-name() = $control-element-name]"/>
    <!-- Get the database entries for the control element. -->
    <xsl:variable name="control-enum-elements" select="$database-control-element/*[local-name() = $control-element-enum-name]"/>
    <!-- Get the database entry for the control element, then get the control attribute value in that entry. -->
    <xsl:variable name="other-control-value" select="$control-enum-elements[@value = $control-element/@value]/@*[local-name() = $control-attribute-name]"/>
    <!-- Make the element with the different values. -->
    <xsl:element name="{$element-name}">
      <!-- Find the entries in the database for the element being prepared. -->
      <xsl:for-each select="$database-element/*[local-name() = $element-enum-name]">
        <!-- Remember the current node for future comparisons. -->
        <xsl:variable name="enum-element" select="."/>
        <!-- Find whether the current entry matches the input value. -->
        <xsl:variable name="matching-element" select="$found-element[@value = $enum-element/@value]"/>
        <!-- Get the control attribute value for the current entry. -->
        <xsl:variable name="this-control-value" select="@*[local-name() = $control-attribute-name]"/>
        <!-- Filter the entry out unless no control element was given, this entry has no control attribute value or...
             if the control attribute value matches the one for the control element. -->
        <xsl:if test="not($has-control-values) or not($this-control-value) or $this-control-value = $other-control-value">
          <!-- Copy the entry. -->
          <xsl:copy>
            <!-- Include all attributes except the price, adding the price only if the item is selected. -->
            <xsl:apply-templates select="@*[local-name() != 'price']"/>
            <!-- Find whether this item is selected. -->
            <xsl:choose>
              <!-- In the current field, the value attribute matches this value. -->
              <xsl:when test="$matching-element">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
                <xsl:apply-templates select="@price"/>
              </xsl:when>
              <!-- This value is the first and no value attribute is specified. -->
              <xsl:when test="not($has-set-values) and count($enum-element/preceding-sibling::*) = 0">
                <xsl:attribute name="value-is-set">true</xsl:attribute>
                <xsl:apply-templates select="@price"/>
              </xsl:when>
              <xsl:otherwise/>
            </xsl:choose>
          </xsl:copy>
        </xsl:if>
      </xsl:for-each>
    </xsl:element>
  </xsl:template>

  <!-- Replicate unknown elements. -->

  <xsl:template match="@*|node()">
    <xsl:copy>
      <xsl:apply-templates select="@*|node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
